/* melodiasAleatorias.c - melodas aleatorias usando distintas distribuciones de probabilidad 
                         (uniforme, lineal, triangular y gaussiana) */
#include <stdio.h>
#include <limits.h>
#include <time.h>
#include <math.h>

#define NUM 50

int uniforme(int rango, int bias);
int lineal(int rango, int bias);
int triangular(int rango, int bias);
int gaussiana(int rango, int bias);

void textoAmidi(char *archivo_texto, char *archivo_midi);
void uso(void);

int main(int argc, char *argv[]){


	int i, velocity = 127;
	int rango, bias; /* notas entre bias y bias+rango-1 */
	float tiempo = 1, duracion = 0.25;
	FILE *fPtr; /* puntero a archivo */
	char dist, *temp="temp.txt"; /* nombre de archivo temporal */

	if (argc != 5){
		uso();
	}
	else{
	  srand(time(NULL));
	  
	  bias = atoi(argv[3]); /* nota inferior */
	  rango = atoi(argv[4]) - atoi(argv[3]) + 1; /* rango = nota superior - nota inferior + 1 */
	  dist = (*argv[2]);
	
	  if ((fPtr = fopen(temp, "w")) == NULL)
		printf("No se pudo crear archivo\n");
	  else{	
	    if ((dist == 'U') || (dist == 'L') || (dist == 'T') || (dist == 'G')){
		for (i = 0; i < NUM; i++){
		  switch (dist){
		    case 'U': 
		      fprintf(fPtr, "note\t%.3f\t%.3f\t%d\t%d\n", tiempo, duracion, uniforme(rango, bias), velocity);
		      break;
		    case 'L': 
		      fprintf(fPtr, "note\t%.3f\t%.3f\t%d\t%d\n", tiempo, duracion, lineal(rango, bias), velocity);
		      break;   
		    case 'T': 
		      fprintf(fPtr, "note\t%.3f\t%.3f\t%d\t%d\n", tiempo, duracion, triangular(rango, bias), velocity);
		      break;   
		    case 'G': 
		      fprintf(fPtr, "note\t%.3f\t%.3f\t%d\t%d\n", tiempo, duracion, gaussiana(rango, bias), velocity);
		      break;    
		  }    
		  tiempo+=duracion;
		}
		fclose(fPtr);
		textoAmidi(temp, argv[1]);
	   }	
	  }
	}	

	return 0;
}

int uniforme(int rango, int bias){
	
	return (rand() % rango) + bias;

}

int lineal(int rango, int bias){
	int u1, u2;
	
	u1 = uniforme(rango, bias);
	u2 = uniforme(rango, bias);
	
	if (u2 < u1)
		u1 = u2;
	
	return u1;
}

int triangular(int rango, int bias){
	int u1, u2;
	
	u1 = uniforme(rango, bias);
	u2 = uniforme(rango, bias);
	
	return (u1 + u2)/2;
}

int gaussiana(int rango, int bias){
	int i, sum = 0;
	int N = 12;
	
	for (i = 0; i < N; i++)
		sum += uniforme(rango, bias);
	
	return sum/N;
}

void textoAmidi(char *archivo_texto, char *archivo_midi){
     
     char comando[PATH_MAX] = "./text2midi ";
     
     /* Se construye el comando a ejecutar */
     strcat(comando, archivo_texto);
     strcat(comando, " ");
     strcat(comando, archivo_midi);

     /* Se hace una llamada al sistema */
     system(comando);
}

void uso(void){
	printf("\nPrograma para generar secuencias de notas aleatorias siguiendo cierta distribucin de probabilidad.\nLas distribuciones disponibles son: uniforme, lineal, triangular y gaussiana\n\n");
	printf("uso: melodiasAleatorias archivo_midi distribucion midi_inicial midi_final\n\n");
	printf("archivo_midi - nombre del archivo midi de salida\n");
	printf("distribucion - distribucion a usar (uniforme: U, lineal: L, triangular: T y gaussiana: G)\n");
	printf("midi_inicial - numero midi inicial del rango a sortear\n");
	printf("midi_final   - numero midi final del rango a sortear\n\n");
	
	printf("Ejemplo: \n");
	printf("./melodiasAleatorias secuencia.mid G 24 127\n");
	printf("Distribucin gaussiana, en el rango de notas midi entre 24 (C1) y 127 (G9).\n\n");

	
}
